@@ -20,151 +20,54 @@
 #include "kmimetype.h"
 #include "kmimetype_p.h"
 #include "kmimetypefactory.h"
+#include "kmimetyperepository_p.h"
 
-#include <kconfig.h>
-#include <kconfiggroup.h>
 #include <kdebug.h>
-#include <kdesktopfile.h>
-#include <kde_file.h>
+#include <kde_file.h> // KDE::stat
+#include <kdeversion.h> // KDE_MAKE_VERSION
 #include <klocale.h>
-#include <kmessage.h>
 #include <kprotocolinfo.h>
 #include <kprotocolinfofactory.h>
 #include <kstandarddirs.h>
-#include <ksycoca.h>
 #include <kurl.h>
 
-#include <QtCore/QMutableSetIterator>
-#include <QtCore/QCharRef>
 #include <QtCore/QFile>
 #include <QtDBus/QtDBus>
 #include <QBuffer>
 
-#include <sys/types.h>
-#include <sys/stat.h>
-
-#include <assert.h>
-#include <errno.h>
-#include <stddef.h>
-#include <unistd.h>
-#include <stdlib.h>
+extern int servicesDebugArea();
 
 template class KSharedPtr<KMimeType>;
 
-static KMimeType::Ptr s_pDefaultMimeType;
-
-static void errorMissingMimeTypes( const QStringList& _types )
-{
-    KMessage::message( KMessage::Error, i18np( "Could not find mime type <resource>%2</resource>",
-                "Could not find mime types:\n<resource>%2</resource>", _types.count(), _types.join("</resource>\n<resource>") ) );
-}
-
-void KMimeTypePrivate::loadInternal( QDataStream& _str )
-{
-    QString oldParentMimeTypeString;
-    _str >> m_lstPatterns >> oldParentMimeTypeString >> m_parentMimeTypes;
-
-    // kde-4.0 compatibility. Remove in kde5.
-    if (!oldParentMimeTypeString.isEmpty() && m_parentMimeTypes.isEmpty())
-        m_parentMimeTypes.append(oldParentMimeTypeString);
-}
-
-/**
- * This function makes sure that the default mime type exists.
- * Not file-static because it needs access to the private KMimeType constructor...
- */
-void KMimeType::buildDefaultType()
-{
-    assert ( !s_pDefaultMimeType );
-    // Try to find the default type
-    KMimeType::Ptr mime = KMimeTypeFactory::self()->
-                          findMimeTypeByName( KMimeType::defaultMimeType() );
-
-    if (mime)
-    {
-        s_pDefaultMimeType = mime;
-    }
-    else
-    {
-        QString defaultMimeType = KMimeType::defaultMimeType();
-        errorMissingMimeTypes( QStringList(defaultMimeType) );
-        QString sDefaultMimeType = KGlobal::dirs()->resourceDirs("xdgdata-mime").first()+defaultMimeType+".xml";
-        s_pDefaultMimeType = new KMimeType( sDefaultMimeType, defaultMimeType, "mime" );
-    }
-}
-
 KMimeType::Ptr KMimeType::defaultMimeTypePtr()
 {
-    if ( !s_pDefaultMimeType ) // we need a default type first
-        buildDefaultType();
-    return s_pDefaultMimeType;
+    return KMimeTypeRepository::self()->defaultMimeTypePtr();
 }
 
 bool KMimeType::isDefault() const
 {
-    return this == defaultMimeTypePtr().data();
+    return name() == defaultMimeType();
 }
 
-/**
- * This function makes sure that vital mime types are installed.
- */
 void KMimeType::checkEssentialMimeTypes()
 {
-    static bool s_bChecked = false;
-
-    if ( s_bChecked ) // already done
-        return;
-    if ( !s_pDefaultMimeType ) // we need a default type first
-        KMimeType::buildDefaultType();
-
-  s_bChecked = true; // must be done before building mimetypes
-
-  // No Mime-Types installed ?
-  // Lets do some rescue here.
-  if ( !KMimeTypeFactory::self()->checkMimeTypes() )
-  {
-    // Note that this messagebox is queued, so it will only be shown once getting back to the event loop
-
-    // No mimetypes installed? Are you setting XDG_DATA_DIRS without including /usr/share in it?
-    KMessage::message( KMessage::Error, i18n( "No mime types installed. Check that shared-mime-info is installed, and that XDG_DATA_DIRS is not set, or includes /usr/share." ) );
-    return; // no point in going any further
-  }
-
-  QStringList missingMimeTypes;
-
-#ifndef Q_OS_WIN
-  if ( !KMimeType::mimeType( "inode/directory" ) )
-    missingMimeTypes.append( "inode/directory" );
-  //if ( !KMimeType::mimeType( "inode/directory-locked" ) )
-  //  missingMimeTypes.append( "inode/directory-locked" );
-  if ( !KMimeType::mimeType( "inode/blockdevice" ) )
-    missingMimeTypes.append( "inode/blockdevice" );
-  if ( !KMimeType::mimeType( "inode/chardevice" ) )
-    missingMimeTypes.append( "inode/chardevice" );
-  if ( !KMimeType::mimeType( "inode/socket" ) )
-    missingMimeTypes.append( "inode/socket" );
-  if ( !KMimeType::mimeType( "inode/fifo" ) )
-    missingMimeTypes.append( "inode/fifo" );
-#endif
-  if ( !KMimeType::mimeType( "application/x-shellscript" ) )
-    missingMimeTypes.append( "application/x-shellscript" );
-  if ( !KMimeType::mimeType( "application/x-executable" ) )
-    missingMimeTypes.append( "application/x-executable" );
-  if ( !KMimeType::mimeType( "application/x-desktop" ) )
-    missingMimeTypes.append( "application/x-desktop" );
-
-  if (!missingMimeTypes.isEmpty())
-    errorMissingMimeTypes(missingMimeTypes);
+    KMimeTypeRepository::self()->checkEssentialMimeTypes();
 }
 
-KMimeType::Ptr KMimeType::mimeType( const QString& _name, FindByNameOption options )
+KMimeType::Ptr KMimeType::mimeType(const QString& name, FindByNameOption options)
 {
-    return KMimeTypeFactory::self()->findMimeTypeByName( _name, options );
+    return KMimeTypeRepository::self()->findMimeTypeByName(name, options);
 }
 
 KMimeType::List KMimeType::allMimeTypes()
 {
-    return KMimeTypeFactory::self()->allMimeTypes();
+    // This could be done faster...
+    KMimeType::List lst;
+    Q_FOREACH(const QString& mimeType, KMimeTypeFactory::self()->allMimeTypes()) {
+        if (!mimeType.startsWith(QLatin1String("x-scheme-handler")))
+            lst.append(KMimeType::mimeType(mimeType));
+    }
+    return lst;
 }
 
 bool KMimeType::isBufferBinaryData(const QByteArray& data)
@@ -185,7 +88,7 @@ static KMimeType::Ptr findFromMode( const QString& path /*only used if is_local_
 {
     if ( is_local_file && (mode == 0 || mode == (mode_t)-1) ) {
         KDE_struct_stat buff;
-        if ( KDE_stat( QFile::encodeName(path), &buff ) != -1 )
+        if ( KDE::stat( path, &buff ) != -1 )
             mode = buff.st_mode;
     }
 
@@ -196,44 +99,49 @@ static KMimeType::Ptr findFromMode( const QString& path /*only used if is_local_
         // are allowed to enter the directory
         if ( is_local_file )
         {
-            if ( access( QFile::encodeName(path), R_OK ) == -1 )
+            if ( KDE::access( path, R_OK ) == -1 )
                 return KMimeType::mimeType( "inode/directory-locked" );
         }
 #endif
-        return KMimeType::mimeType( "inode/directory" );
+        return KMimeType::mimeType( QLatin1String("inode/directory") );
     }
     if ( S_ISCHR( mode ) )
-        return KMimeType::mimeType( "inode/chardevice" );
+        return KMimeType::mimeType( QLatin1String("inode/chardevice") );
     if ( S_ISBLK( mode ) )
-        return KMimeType::mimeType( "inode/blockdevice" );
+        return KMimeType::mimeType( QLatin1String("inode/blockdevice") );
     if ( S_ISFIFO( mode ) )
-        return KMimeType::mimeType( "inode/fifo" );
+        return KMimeType::mimeType( QLatin1String("inode/fifo") );
     if ( S_ISSOCK( mode ) )
-        return KMimeType::mimeType( "inode/socket" );
+        return KMimeType::mimeType( QLatin1String("inode/socket") );
 #ifdef Q_OS_WIN
     // FIXME: distinguish between mounted & unmounted
     int size = path.size();
     if ( size == 2 || size == 3 ) {
+    //GetDriveTypeW is not defined in wince
+#ifndef _WIN32_WCE
         unsigned int type = GetDriveTypeW( (LPCWSTR) path.utf16() );
         switch( type ) {
             case DRIVE_REMOVABLE:
-                return KMimeType::mimeType( "media/floppy_mounted" );
+                return KMimeType::mimeType( QLatin1String("media/floppy_mounted") );
             case DRIVE_FIXED:
-                return KMimeType::mimeType( "media/hdd_mounted" );
+                return KMimeType::mimeType( QLatin1String("media/hdd_mounted") );
             case DRIVE_REMOTE:
-                return KMimeType::mimeType( "media/smb_mounted" );
+                return KMimeType::mimeType( QLatin1String("media/smb_mounted") );
             case DRIVE_CDROM:
-                return KMimeType::mimeType( "media/cdrom_mounted" );
+                return KMimeType::mimeType( QLatin1String("media/cdrom_mounted") );
             case DRIVE_RAMDISK:
-                return KMimeType::mimeType( "media/hdd_mounted" );
+                return KMimeType::mimeType( QLatin1String("media/hdd_mounted") );
             default:
                 break;
         };
+#else
+        return KMimeType::mimeType( QLatin1String("media/hdd_mounted") );
+#endif
     }
 #endif
     // remote executable file? stop here (otherwise findFromContent can do that better for local files)
     if ( !is_local_file && S_ISREG( mode ) && ( mode & ( S_IXUSR | S_IXGRP | S_IXOTH ) ) )
-        return KMimeType::mimeType( "application/x-executable" );
+        return KMimeType::mimeType( QLatin1String("application/x-executable") );
 
     return KMimeType::Ptr();
 }
@@ -274,7 +182,7 @@ KMimeType::Ptr KMimeType::findByUrlHelper( const KUrl& _url, mode_t mode,
                                            int* accuracy )
 {
     checkEssentialMimeTypes();
-    const QString path = _url.path();
+    const QString path = is_local_file ? _url.toLocalFile() : _url.path();
 
     if (accuracy)
         *accuracy = 100;
@@ -286,16 +194,25 @@ KMimeType::Ptr KMimeType::findByUrlHelper( const KUrl& _url, mode_t mode,
 
     // First try to find out by looking at the filename (if there's one)
     const QString fileName( _url.fileName() );
-    QList<KMimeType::Ptr> mimeList;
-    if ( !fileName.isEmpty() && !path.endsWith( '/' ) ) {
+    QStringList mimeList;
+    if ( !fileName.isEmpty() && !path.endsWith( QLatin1Char('/') ) ) {
         // and if we can trust it (e.g. don't trust *.pl over HTTP, could be anything)
         if ( is_local_file || _url.hasSubUrl() || // Explicitly trust suburls
              KProtocolInfo::determineMimetypeFromExtension( _url.protocol() ) ) {
-            mimeList = KMimeTypeFactory::self()->findFromFileName( fileName );
+            mimeList = KMimeTypeRepository::self()->findFromFileName( fileName );
             // Found one glob match exactly: OK, use that.
             // We disambiguate multiple glob matches by sniffing, below.
             if ( mimeList.count() == 1 ) {
-                return mimeList.first();
+                const QString selectedMime = mimeList.at(0);
+                KMimeType::Ptr mime = mimeType(selectedMime);
+                if (!mime) {
+                    // #265188 - this can happen when an old globs file is lying around after
+                    // the packages xml file was removed.
+                    kWarning() << "Glob file refers to" << selectedMime << "but this mimetype does not exist!";
+                    mimeList.clear();
+                } else {
+                    return mime;
+                }
             }
         }
     }
@@ -310,11 +227,10 @@ KMimeType::Ptr KMimeType::findByUrlHelper( const KUrl& _url, mode_t mode,
     QByteArray beginning;
     if ( device ) {
         int magicAccuracy;
-        KMimeType::Ptr mime = KMimeTypeFactory::self()->findFromContent(
-            device, KMimeTypeFactory::AllRules, &magicAccuracy, beginning );
+        KMimeType::Ptr mime = KMimeTypeRepository::self()->findFromContent(device, &magicAccuracy, beginning);
         // mime can't be 0, except in case of install problems.
         // However we get magicAccuracy==0 for octet-stream, i.e. no magic match found.
-        //kDebug(7009) << "findFromContent said" << (mime?mime->name():QString()) << "with accuracy" << magicAccuracy;
+        //kDebug(servicesDebugArea()) << "findFromContent said" << (mime?mime->name():QString()) << "with accuracy" << magicAccuracy;
         if (mime && magicAccuracy > 0) {
 
             // Disambiguate conflicting extensions (if magic found something and the magicrule was <80)
@@ -322,9 +238,10 @@ KMimeType::Ptr KMimeType::findByUrlHelper( const KUrl& _url, mode_t mode,
                 // "for glob_match in glob_matches:"
                 // "if glob_match is subclass or equal to sniffed_type, use glob_match"
                 const QString sniffedMime = mime->name();
-                foreach(const KMimeType::Ptr &mimeFromPattern, mimeList) {
-                    //kDebug(7009) << "sniffedMime=" << sniffedMime << "mimeFromPattern=" << mimeFromPattern->name();
-                    if (mimeFromPattern->is(sniffedMime)) {
+                foreach(const QString &m, mimeList) {
+                    KMimeType::Ptr mimeFromPattern = KMimeType::mimeType(m);
+                    //kDebug(servicesDebugArea()) << "sniffedMime=" << sniffedMime << "mimeFromPattern=" << mimeFromPattern->name();
+                    if (mimeFromPattern && mimeFromPattern->is(sniffedMime)) {
                         // We have magic + pattern pointing to this, so it's a pretty good match
                         if (accuracy)
                             *accuracy = 100;
@@ -346,7 +263,15 @@ KMimeType::Ptr KMimeType::findByUrlHelper( const KUrl& _url, mode_t mode,
         if (accuracy)
             *accuracy = 20;
         // We have to pick one...
-        return mimeList.first();
+        // At least make this deterministic
+        qSort(mimeList.begin(), mimeList.end());
+        Q_FOREACH(const QString& mimeName, mimeList) {
+            KMimeType::Ptr mime = mimeType(mimeName);
+            if (!mime)
+                kWarning() << "Glob file refers to" << mimeName << "but this mimetype does not exist!";
+            else
+                return mime;
+        }
     }
 
     // Find a fallback from the protocol
@@ -364,7 +289,7 @@ KMimeType::Ptr KMimeType::findByUrlHelper( const KUrl& _url, mode_t mode,
         if (mime)
             return mime;
     }
-    if ( path.endsWith( '/' ) || path.isEmpty() ) {
+    if ( path.endsWith( QLatin1Char('/') ) || path.isEmpty() ) {
         // We have no filename at all. Maybe the protocol has a setting for
         // which mimetype this means (e.g. directory).
         // For HTTP (def==defaultMimeType()) we don't assume anything,
@@ -394,7 +319,7 @@ KMimeType::Ptr KMimeType::findByUrl( const KUrl& url, mode_t mode,
     if ( !is_local_file && url.isLocalFile() )
         is_local_file = true;
     if (is_local_file && !fast_mode) {
-        QFile file(url.path());
+        QFile file(url.toLocalFile());
         return findByUrlHelper(url, mode, is_local_file, &file, accuracy);
     }
     return findByUrlHelper(url, mode, is_local_file, 0, accuracy);
@@ -417,10 +342,18 @@ KMimeType::Ptr KMimeType::findByNameAndContent( const QString& name, const QByte
     return findByUrlHelper(url, mode, false, &buffer, accuracy);
 }
 
+KMimeType::Ptr KMimeType::findByNameAndContent( const QString& name, QIODevice* device,
+                                                mode_t mode, int* accuracy )
+{
+    KUrl url;
+    url.setPath(name);
+    return findByUrlHelper(url, mode, false, device, accuracy);
+}
+
 QString KMimeType::extractKnownExtension(const QString &fileName)
 {
     QString pattern;
-    KMimeTypeFactory::self()->findFromFileName( fileName, &pattern );
+    KMimeTypeRepository::self()->findFromFileName( fileName, &pattern );
     return pattern;
 }
 
@@ -429,8 +362,13 @@ KMimeType::Ptr KMimeType::findByContent( const QByteArray &data, int *accuracy )
     QBuffer buffer(const_cast<QByteArray *>(&data));
     buffer.open(QIODevice::ReadOnly);
     QByteArray cache;
-    return KMimeTypeFactory::self()->findFromContent(
-        &buffer, KMimeTypeFactory::AllRules, accuracy, cache );
+    return KMimeTypeRepository::self()->findFromContent(&buffer, accuracy, cache);
+}
+
+KMimeType::Ptr KMimeType::findByContent( QIODevice* device, int* accuracy )
+{
+    QByteArray cache;
+    return KMimeTypeRepository::self()->findFromContent(device, accuracy, cache);
 }
 
 KMimeType::Ptr KMimeType::findByFileContent( const QString &fileName, int *accuracy )
@@ -452,8 +390,7 @@ KMimeType::Ptr KMimeType::findByFileContent( const QString &fileName, int *accur
     }
 
     QByteArray cache;
-    return KMimeTypeFactory::self()->findFromContent(
-        &device, KMimeTypeFactory::AllRules, accuracy, cache );
+    return KMimeTypeRepository::self()->findFromContent(&device, accuracy, cache);
 }
 
 bool KMimeType::isBinaryData( const QString &fileName )
@@ -490,16 +427,16 @@ KMimeType::KMimeType( QDataStream& _str, int offset )
 void KMimeTypePrivate::save( QDataStream& _str )
 {
     KServiceTypePrivate::save( _str );
-    // Warning adding/removing fields here involves a binary incompatible change - update version
-    // number in ksycoca.h
-    _str << m_lstPatterns << QString() << m_parentMimeTypes;
+    // Warning adding fields here involves a binary incompatible change - update version
+    // number in ksycoca.h. Never remove fields.
+    _str << m_lstPatterns << QString() << QStringList() << m_iconName;
 }
 
 QVariant KMimeTypePrivate::property( const QString& _name ) const
 {
-    if ( _name == "Patterns" )
+    if ( _name == QLatin1String("Patterns") )
         return QVariant( m_lstPatterns );
-    if ( _name == "Icon" )
+    if ( _name == QLatin1String("Icon") )
         return QVariant( iconName(KUrl()) );
 
     return KServiceTypePrivate::property( _name );
@@ -508,8 +445,8 @@ QVariant KMimeTypePrivate::property( const QString& _name ) const
 QStringList KMimeTypePrivate::propertyNames() const
 {
     QStringList res = KServiceTypePrivate::propertyNames();
-    res.append( "Patterns" );
-    res.append( "Icon" );
+    res.append( QString::fromLatin1("Patterns") );
+    res.append( QString::fromLatin1("Icon") );
     return res;
 }
 
@@ -529,7 +466,7 @@ QString KMimeType::iconNameForUrl( const KUrl & _url, mode_t mode )
     QString i = mimeTypeIcon;
 
     // if we don't find an icon, maybe we can use the one for the protocol
-    if ( i == unknown || i.isEmpty() || mt == defaultMimeTypePtr()
+    if ( i == unknown || i.isEmpty() || mt->name() == defaultMimeType()
         // and for the root of the protocol (e.g. trash:/) the protocol icon has priority over the mimetype icon
         || _url.path().length() <= 1 )
     {
@@ -547,22 +484,15 @@ QString KMimeType::iconNameForUrl( const KUrl & _url, mode_t mode )
 
 QString KMimeType::favIconForUrl( const KUrl& url )
 {
-    // this method will be called quite often, so better not read the config
-    // again and again.
-    static bool useFavIcons = true;
-    static bool check = true;
-    if ( check ) {
-        check = false;
-        KConfigGroup cg( KGlobal::config(), "HTML Settings" );
-        useFavIcons = cg.readEntry("EnableFavicon", true);
-    }
-
-    if ( url.isLocalFile() || !url.protocol().startsWith("http")
-         || !useFavIcons )
+    if (url.isLocalFile()
+        || !url.protocol().startsWith(QLatin1String("http"))
+        || !KMimeTypeRepository::self()->useFavIcons())
         return QString();
 
-    QDBusInterface kded( "org.kde.kded", "/modules/favicons", "org.kde.FavIcon" );
-    QDBusReply<QString> result = kded.call( "iconForUrl", url.url() );
+    QDBusInterface kded( QString::fromLatin1("org.kde.kded"),
+                         QString::fromLatin1("/modules/favicons"),
+                         QString::fromLatin1("org.kde.FavIcon") );
+    QDBusReply<QString> result = kded.call( QString::fromLatin1("iconForUrl"), url.url() );
     return result;              // default is QString()
 }
 
@@ -572,42 +502,27 @@ QString KMimeType::comment( const KUrl &url) const
     return d->comment(url);
 }
 
+#ifndef KDE_NO_DEPRECATED
 QString KMimeType::parentMimeType() const
 {
-    Q_D(const KMimeType);
-
-    if (!d->m_parentMimeTypes.isEmpty())
-        return d->m_parentMimeTypes.first();
-    return d->fallbackParent();
-}
-
-QString KMimeTypePrivate::fallbackParent() const
-{
-    const QString myGroup = m_strName.left(m_strName.indexOf('/'));
-    // All text/* types are subclasses of text/plain.
-    if (myGroup == "text" && m_strName != "text/plain")
-        return "text/plain";
-    // All real-file mimetypes implicitly derive from application/octet-stream
-    if (myGroup != "inode" &&
-        // kde extensions
-        myGroup != "all" && myGroup != "fonts" && myGroup != "print" && myGroup != "uri"
-        && m_strName != "application/octet-stream") {
-        return "application/octet-stream";
-    }
+    const QStringList parents = parentMimeTypes();
+    if (!parents.isEmpty())
+        return parents.first();
     return QString();
 }
+#endif
 
 bool KMimeTypePrivate::inherits(const QString& mime) const
 {
-    if (m_strName == mime) {
-        return true;
-    }
-    foreach( const QString& parent, parentMimeTypes() ) {
-        KMimeType::Ptr parentMime = KMimeTypeFactory::self()->findMimeTypeByName(parent);
-        if (!parentMime) // error
-            return false;
-        if (parentMime->d_func()->inherits(mime)) // recurse
+    QStack<QString> toCheck;
+    toCheck.push(m_strName);
+    while (!toCheck.isEmpty()) {
+        const QString current = toCheck.pop();
+        if (current == mime)
             return true;
+        Q_FOREACH(const QString& parent, KMimeTypeRepository::self()->parents(current)) {
+            toCheck.push(parent);
+        }
     }
     return false;
 }
@@ -617,32 +532,19 @@ bool KMimeType::is( const QString& mimeTypeName ) const
     Q_D(const KMimeType);
     if (name() == mimeTypeName)
         return true;
-    QString mime = KMimeTypeFactory::self()->resolveAlias(mimeTypeName);
-    if (mime.isEmpty())
-        mime = mimeTypeName;
+    const QString mime = KMimeTypeRepository::self()->canonicalName(mimeTypeName);
     return d->inherits(mime);
 }
 
 QStringList KMimeType::parentMimeTypes() const
 {
     Q_D(const KMimeType);
-    return d->parentMimeTypes();
+    return KMimeTypeRepository::self()->parents(d->m_strName);
 }
 
-QStringList KMimeTypePrivate::parentMimeTypes() const
+static void collectParentMimeTypes(const QString& mime, QStringList& allParents)
 {
-    QStringList parents = m_parentMimeTypes;
-    if (parents.isEmpty()) {
-        const QString myParent = fallbackParent();
-        if (!myParent.isEmpty())
-            parents.append(myParent);
-    }
-    return parents;
-}
-
-void KMimeTypePrivate::collectParentMimeTypes(QStringList& allParents) const
-{
-    QStringList parents = parentMimeTypes();
+    QStringList parents = KMimeTypeRepository::self()->parents(mime);
     Q_FOREACH(const QString& parent, parents) {
         // I would use QSet, but since order matters I better not
         if (!allParents.contains(parent))
@@ -651,19 +553,18 @@ void KMimeTypePrivate::collectParentMimeTypes(QStringList& allParents) const
     // We want a breadth-first search, so that the least-specific parent (octet-stream) is last
     // This means iterating twice, unfortunately.
     Q_FOREACH(const QString& parent, parents) {
-        KMimeType::Ptr parentMime = KMimeTypeFactory::self()->findMimeTypeByName(parent);
-        if (parentMime)
-            parentMime->d_func()->collectParentMimeTypes(allParents);
+        collectParentMimeTypes(parent, allParents);
     }
 }
 
 QStringList KMimeType::allParentMimeTypes() const
 {
+    Q_D(const KMimeType);
     QStringList allParents;
-    const QString canonical = KMimeTypeFactory::self()->resolveAlias(name());
+    const QString canonical = KMimeTypeRepository::self()->resolveAlias(name());
     if (!canonical.isEmpty())
         allParents.append(canonical);
-    d_func()->collectParentMimeTypes(allParents);
+    collectParentMimeTypes(d->m_strName, allParents);
     return allParents;
 }
 
@@ -683,26 +584,175 @@ QString KMimeType::iconName( const KUrl& url) const
 QStringList KMimeType::patterns() const
 {
     Q_D(const KMimeType);
+    d->ensureXmlDataLoaded();
     return d->m_lstPatterns;
 }
 
-void KMimeType::setPatterns(const QStringList& patterns)
+// loads comment, icon, mainPattern, m_lstPatterns
+void KMimeTypePrivate::ensureXmlDataLoaded() const
 {
-    Q_D(KMimeType);
-    d->m_lstPatterns = patterns;
+    if (m_xmlDataLoaded)
+        return;
+
+    m_xmlDataLoaded = true;
+
+    const QString file = m_strName + QLatin1String(".xml");
+    const QStringList mimeFiles = KGlobal::dirs()->findAllResources("xdgdata-mime", file);
+    if (mimeFiles.isEmpty()) {
+        kWarning() << "No file found for" << file << ", even though the file appeared in a directory listing.";
+        kWarning() << "Either it was just removed, or the directory doesn't have executable permission...";
+        kWarning() << KGlobal::dirs()->resourceDirs("xdgdata-mime");
+        return;
+    }
+
+    QString comment;
+    QString mainPattern;
+    const QStringList languageList = KGlobal::locale()->languageList();
+    QString preferredLanguage = languageList.first();
+    QMap<QString, QString> commentsByLanguage;
+
+    QListIterator<QString> mimeFilesIter(mimeFiles);
+    mimeFilesIter.toBack();
+    while (mimeFilesIter.hasPrevious()) { // global first, then local.
+        const QString fullPath = mimeFilesIter.previous();
+        QFile qfile(fullPath);
+        if (!qfile.open(QFile::ReadOnly))
+            continue;
+
+        QXmlStreamReader xml(&qfile);
+        if (xml.readNextStartElement()) {
+            if (xml.name() != "mime-type") {
+                continue;
+            }
+            const QString name = xml.attributes().value(QLatin1String("type")).toString();
+            if (name.isEmpty())
+                continue;
+            if (name != m_strName) {
+                kWarning() << "Got name" << name << "in file" << file << "expected" << m_strName;
+            }
+
+            while (xml.readNextStartElement()) {
+                const QStringRef tag = xml.name();
+                if (tag == "comment") {
+                    QString lang = xml.attributes().value(QLatin1String("xml:lang")).toString();
+                    const QString text = xml.readElementText();
+                    if (lang.isEmpty()) {
+                        lang = QLatin1String("en_US");
+                    }
+                    if (lang == preferredLanguage) {
+                        comment = text;
+                    } else {
+                        commentsByLanguage.insert(lang, text);
+                    }
+                    continue; // we called readElementText, so we're at the EndElement already.
+                } else if (tag == "icon") { // as written out by shared-mime-info >= 0.40
+                    m_iconName = xml.attributes().value(QLatin1String("name")).toString();
+                } else if (tag == "glob-deleteall") { // as written out by shared-mime-info >= 0.70
+                    mainPattern.clear();
+                    m_lstPatterns.clear();
+                } else if (tag == "glob") { // as written out by shared-mime-info >= 0.70
+                    const QString pattern = xml.attributes().value(QLatin1String("pattern")).toString();
+                    if (mainPattern.isEmpty() && pattern.startsWith(QLatin1Char('*'))) {
+                        mainPattern = pattern;
+                    }
+                    if (!m_lstPatterns.contains(pattern))
+                        m_lstPatterns.append(pattern);
+                }
+                xml.skipCurrentElement();
+            }
+            if (xml.name() != "mime-type") {
+                kFatal() << "Programming error in KMimeType XML loading, please create a bug report on http://bugs.kde.org and attach the file" << fullPath;
+            }
+        }
+    }
+
+    if (comment.isEmpty()) {
+        Q_FOREACH(const QString& lang, languageList) {
+            const QString comm = commentsByLanguage.value(lang);
+            if (!comm.isEmpty()) {
+                comment = comm;
+                break;
+            }
+            const int pos = lang.indexOf(QLatin1Char('_'));
+            if (pos != -1) {
+                // "pt_BR" not found? try just "pt"
+                const QString shortLang = lang.left(pos);
+                const QString comm = commentsByLanguage.value(shortLang);
+                if (!comm.isEmpty()) {
+                    comment = comm;
+                    break;
+                }
+            }
+        }
+        if (comment.isEmpty()) {
+            kWarning() << "Missing <comment> field in" << file;
+        }
+    }
+    m_strComment = comment;
+
+    const bool globsInXml = (KMimeType::sharedMimeInfoVersion() >= KDE_MAKE_VERSION(0, 70, 0));
+    if (globsInXml) {
+        if (!mainPattern.isEmpty() && m_lstPatterns.first() != mainPattern) {
+            // ensure it's first in the list of patterns
+            m_lstPatterns.removeAll(mainPattern);
+            m_lstPatterns.prepend(mainPattern);
+        }
+    } else {
+        // Fallback: get the patterns from the globs file
+        m_lstPatterns = KMimeTypeRepository::self()->patternsForMimetype(m_strName);
+    }
 }
 
-void KMimeType::setParentMimeType(const QString& parent)
+QString KMimeType::userSpecifiedIconName() const
 {
-    Q_D(KMimeType);
-    // kbuildmimetypefactory calls this multiple times, for each parent mimetype
-    d->m_parentMimeTypes.append(parent);
+    Q_D(const KMimeType);
+    d->ensureXmlDataLoaded();
+    return d->m_iconName;
 }
 
-void KMimeType::internalClearData()
+int KMimeType::sharedMimeInfoVersion()
 {
-    Q_D(KMimeType);
-    // Clear the data that KBuildMimeTypeFactory is going to refill - and only that data.
-    d->m_parentMimeTypes.clear();
-    d->m_lstPatterns.clear();
+    return KMimeTypeRepository::self()->sharedMimeInfoVersion();
+}
+
+QString KMimeType::mainExtension() const
+{
+    Q_D(const KMimeType);
+
+#if 1 // HACK START - can be removed once shared-mime-info >= 0.70 is used/required.
+    // The idea was: first usable pattern from m_lstPatterns.
+    // But update-mime-database makes a mess of the order of the patterns,
+    // because it uses a hash internally.
+    static const struct { const char* mime; const char* extension; } s_hardcodedMimes[] = {
+        { "text/plain", ".txt" } };
+    if (d->m_lstPatterns.count() > 1) {
+        const QByteArray me = name().toLatin1();
+        for (uint i = 0; i < sizeof(s_hardcodedMimes)/sizeof(*s_hardcodedMimes); ++i) {
+            if (me == s_hardcodedMimes[i].mime)
+                return QString::fromLatin1(s_hardcodedMimes[i].extension);
+        }
+    }
+#endif // HACK END
+
+     Q_FOREACH(const QString& pattern, patterns()) {
+        // Skip if if looks like: README or *. or *.*
+        // or *.JP*G or *.JP?
+        if (pattern.startsWith(QLatin1String("*.")) &&
+            pattern.length() > 2 &&
+            pattern.indexOf(QLatin1Char('*'), 2) < 0 && pattern.indexOf(QLatin1Char('?'), 2) < 0) {
+            return pattern.mid(1);
+        }
+    }
+    // TODO we should also look into the parent mimetype's patterns, no?
+    return QString();
+}
+
+bool KMimeType::matchFileName( const QString &filename, const QString &pattern )
+{
+    return KMimeTypeRepository::matchFileName( filename, pattern );
+}
+
+int KMimeTypePrivate::serviceOffersOffset() const
+{
+    return KMimeTypeFactory::self()->serviceOffersOffset(name());
 }
