@@ -364,7 +364,7 @@ int KCategorizedView::Private::highestElementInLastRow(const Block &block) const
 {
     //Find the highest element in the last row
     const QModelIndex lastIndex = proxyModel->index(block.firstIndex.row() + block.items.count() - 1, q->modelColumn(), q->rootIndex());
-    QRect prevRect = q->visualRect(lastIndex);
+    const QRect prevRect = q->visualRect(lastIndex);
     int res = prevRect.height();
     QModelIndex prevIndex = proxyModel->index(lastIndex.row() - 1, q->modelColumn(), q->rootIndex());
     if (!prevIndex.isValid()) {
@@ -405,22 +405,30 @@ void KCategorizedView::Private::leftToRightVisualRect(const QModelIndex &index,
     if (hasGrid()) {
         const int relativeRow = index.row() - firstIndexRow;
         const int maxItemsPerRow = qMax(viewportWidth() / q->gridSize().width(), 1);
-        item.topLeft.rx() = (relativeRow % maxItemsPerRow) * q->gridSize().width() + blockPos.x() + categoryDrawer->leftMargin();
+        if (q->layoutDirection() == Qt::LeftToRight) {
+            item.topLeft.rx() = (relativeRow % maxItemsPerRow) * q->gridSize().width() + blockPos.x() + categoryDrawer->leftMargin();
+        } else {
+            item.topLeft.rx() = viewportWidth() - ((relativeRow % maxItemsPerRow) + 1) * q->gridSize().width() + categoryDrawer->leftMargin() + categorySpacing;
+        }
         item.topLeft.ry() = (relativeRow / maxItemsPerRow) * q->gridSize().height();
     } else {
         if (q->uniformItemSizes()) {
             const int relativeRow = index.row() - firstIndexRow;
             const QSize itemSize = q->sizeHintForIndex(index);
             const int maxItemsPerRow = qMax((viewportWidth() - q->spacing()) / (itemSize.width() + q->spacing()), 1);
-            item.topLeft.rx() = (relativeRow % maxItemsPerRow) * itemSize.width() + blockPos.x() + categoryDrawer->leftMargin();
+            if (q->layoutDirection() == Qt::LeftToRight) {
+                item.topLeft.rx() = (relativeRow % maxItemsPerRow) * itemSize.width() + blockPos.x() + categoryDrawer->leftMargin();
+            } else {
+                item.topLeft.rx() = viewportWidth() - (relativeRow % maxItemsPerRow) * itemSize.width() + categoryDrawer->leftMargin() + categorySpacing;
+            }
             item.topLeft.ry() = (relativeRow / maxItemsPerRow) * itemSize.height();
         } else {
+            const QSize currSize = q->sizeHintForIndex(index);
             if (index != block.firstIndex) {
                 const int viewportW = viewportWidth() - q->spacing();
                 QModelIndex prevIndex = proxyModel->index(index.row() - 1, q->modelColumn(), q->rootIndex());
                 QRect prevRect = q->visualRect(prevIndex);
                 prevRect = mapFromViewport(prevRect);
-                const QSize currSize = q->sizeHintForIndex(index);
                 if ((prevRect.bottomRight().x() + 1) + currSize.width() - blockPos.x() + q->spacing()  > viewportW) {
                     // we have to check the whole previous row, and see which one was the
                     // highest.
@@ -437,19 +445,30 @@ void KCategorizedView::Private::leftToRightVisualRect(const QModelIndex &index,
                             break;
                         }
                     }
-                    item.topLeft.rx() = categoryDrawer->leftMargin() + blockPos.x() + q->spacing();
+                    if (q->layoutDirection() == Qt::LeftToRight) {
+                        item.topLeft.rx() = categoryDrawer->leftMargin() + blockPos.x() + q->spacing();
+                    } else {
+                        item.topLeft.rx() = viewportWidth() - currSize.width() + categoryDrawer->leftMargin() + categorySpacing;
+                    }
                     item.topLeft.ry() = (prevRect.bottomRight().y() + 1) + q->spacing() - blockPos.y();
                 } else {
-                    item.topLeft.rx() = (prevRect.bottomRight().x() + 1) + q->spacing();
+                    if (q->layoutDirection() == Qt::LeftToRight) {
+                        item.topLeft.rx() = (prevRect.bottomRight().x() + 1) + q->spacing();
+                    } else {
+                        item.topLeft.rx() = (prevRect.bottomLeft().x() - 1) - q->spacing() - item.size.width() + categoryDrawer->leftMargin() + categorySpacing;
+                    }
                     item.topLeft.ry() = prevRect.topLeft().y() - blockPos.y();
                 }
             } else {
-                item.topLeft.rx() = blockPos.x() + categoryDrawer->leftMargin() + q->spacing();
+                if (q->layoutDirection() == Qt::LeftToRight) {
+                    item.topLeft.rx() = blockPos.x() + categoryDrawer->leftMargin() + q->spacing();
+                } else {
+                    item.topLeft.rx() = viewportWidth() - currSize.width() + categoryDrawer->leftMargin() + categorySpacing;
+                }
                 item.topLeft.ry() = q->spacing();
             }
         }
     }
-
     item.size = q->sizeHintForIndex(index);
 }
 
@@ -473,7 +492,6 @@ void KCategorizedView::Private::topToBottomVisualRect(const QModelIndex &index,
                 QModelIndex prevIndex = proxyModel->index(index.row() - 1, q->modelColumn(), q->rootIndex());
                 QRect prevRect = q->visualRect(prevIndex);
                 prevRect = mapFromViewport(prevRect);
-                const QSize currSize = q->sizeHintForIndex(index);
                 item.topLeft.rx() = blockPos.x() + categoryDrawer->leftMargin() + q->spacing();
                 item.topLeft.ry() = (prevRect.bottomRight().y() + 1) + q->spacing() - blockPos.y();
             } else {
@@ -482,7 +500,6 @@ void KCategorizedView::Private::topToBottomVisualRect(const QModelIndex &index,
             }
         }
     }
-
     item.size = q->sizeHintForIndex(index);
     item.size.setWidth(viewportWidth());
 }
@@ -727,7 +744,10 @@ QModelIndex KCategorizedView::indexAt(const QPoint &point) const
         const QModelIndex index = d->proxyModel->index(middle, modelColumn(), rootIndex());
         QRect rect = visualRect(index);
         const int verticalOff = verticalOffset();
-        const int horizontalOff = horizontalOffset();
+        int horizontalOff = horizontalOffset();
+        if (layoutDirection() == Qt::RightToLeft) {
+            horizontalOff *= -1;
+        }
         rect.topLeft().ry() += verticalOff;
         rect.topLeft().rx() += horizontalOff;
         rect.bottomRight().ry() += verticalOff;
@@ -738,15 +758,19 @@ QModelIndex KCategorizedView::indexAt(const QPoint &point) const
             }
             return QModelIndex();
         }
+        bool directionCondition;
+        if (layoutDirection() == Qt::LeftToRight) {
+            directionCondition = point.x() > rect.bottomRight().x();
+        } else {
+            directionCondition = point.x() < rect.bottomLeft().x();
+        }
         if (point.y() > rect.bottomRight().y() ||
-            (point.y() > rect.topLeft().y() && point.y() < rect.bottomRight().y() &&
-             point.x() > rect.bottomRight().x())) {
+            (point.y() > rect.topLeft().y() && point.y() < rect.bottomRight().y() && directionCondition)) {
             bottom = middle + 1;
         } else {
             top = middle - 1;
         }
     }
-
     return QModelIndex();
 }
 
@@ -844,6 +868,7 @@ void KCategorizedView::paintEvent(QPaintEvent *event)
                 option.state |= (index == d->hoveredIndex) ? QStyle::State_MouseOver
                                                            : QStyle::State_None;
             }
+
             itemDelegate(index)->paint(&p, option, index);
             ++i;
         }
@@ -1327,6 +1352,29 @@ void KCategorizedView::rowsAboutToBeRemoved(const QModelIndex &parent,
 void KCategorizedView::updateGeometries()
 {
     const int oldVerticalOffset = verticalOffset();
+    const Qt::ScrollBarPolicy verticalP = verticalScrollBarPolicy(), horizontalP = horizontalScrollBarPolicy();
+
+    //BEGIN bugs 213068, 287847 ------------------------------------------------------------
+    /*
+     * QListView::updateGeometries() has it's own opinion on whether the scrollbars should be visible (valid range) or not
+     * and triggers a (sometimes additionally timered) resize through ::layoutChildren()
+     * http://qt.gitorious.org/qt/qt/blobs/4.7/src/gui/itemviews/qlistview.cpp#line1499
+     * (the comment above the main block isn't all accurate, layoutChldren is called regardless of the policy)
+     *
+     * As a result QListView and KCategorizedView occasionally started a race on the scrollbar visibility, effectively blocking the UI
+     * So we prevent QListView from having an own opinion on the scrollbar visibility by
+     * fixing it before calling the baseclass QListView::updateGeometries()
+     *
+     * Since the implicit show/hide by the followin range setting will cause further resizes if the policy is Qt::ScrollBarAsNeeded
+     * we keep it static until we're done, then restore the original value and ultimately change the scrollbar visibility ourself.
+     */
+    if (d->isCategorized()) { // important! - otherwise we'd pollute the setting if the view is initially not categorized
+        setVerticalScrollBarPolicy((verticalP == Qt::ScrollBarAlwaysOn || verticalScrollBar()->isVisibleTo(this)) ?
+                                                                            Qt::ScrollBarAlwaysOn : Qt::ScrollBarAlwaysOff);
+        setHorizontalScrollBarPolicy((horizontalP == Qt::ScrollBarAlwaysOn || horizontalScrollBar()->isVisibleTo(this)) ?
+                                                                            Qt::ScrollBarAlwaysOn : Qt::ScrollBarAlwaysOff);
+    }
+    //END bugs 213068, 287847 --------------------------------------------------------------
 
     QListView::updateGeometries();
 
@@ -1337,6 +1385,8 @@ void KCategorizedView::updateGeometries()
     const int rowCount = d->proxyModel->rowCount();
     if (!rowCount) {
         verticalScrollBar()->setRange(0, 0);
+        // unconditional, see function end todo
+        horizontalScrollBar()->setRange(0, 0);
         return;
     }
 
@@ -1375,6 +1425,19 @@ void KCategorizedView::updateGeometries()
     //      (think how to draw categories), we would have to take care of the horizontal scroll bar too.
     //      In theory, as KCategorizedView has been designed, there is no need of horizontal scroll bar.
     horizontalScrollBar()->setRange(0, 0);
+
+    //BEGIN bugs 213068, 287847 ------------------------------------------------------------
+    // restoring values from above ...
+    setVerticalScrollBarPolicy(verticalP);
+    setHorizontalScrollBarPolicy(horizontalP);
+    // ... and correct the visibility
+    bool validRange = verticalScrollBar()->maximum() != verticalScrollBar()->minimum();
+    if (verticalP == Qt::ScrollBarAsNeeded && (verticalScrollBar()->isVisibleTo(this) != validRange))
+        verticalScrollBar()->setVisible(validRange);
+    validRange = horizontalScrollBar()->maximum() > horizontalScrollBar()->minimum();
+    if (horizontalP == Qt::ScrollBarAsNeeded && (horizontalScrollBar()->isVisibleTo(this) != validRange))
+        horizontalScrollBar()->setVisible(validRange);
+    //END bugs 213068, 287847 --------------------------------------------------------------
 }
 
 void KCategorizedView::currentChanged(const QModelIndex &current,
@@ -1429,6 +1492,7 @@ void KCategorizedView::rowsInserted(const QModelIndex &parent,
     d->rowsInserted(parent, start, end);
 }
 
+#ifndef KDE_NO_DEPRECATED
 void KCategorizedView::rowsInsertedArtifficial(const QModelIndex &parent,
                                                int start,
                                                int end)
@@ -1437,7 +1501,9 @@ void KCategorizedView::rowsInsertedArtifficial(const QModelIndex &parent,
     Q_UNUSED(start);
     Q_UNUSED(end);
 }
+#endif
 
+#ifndef KDE_NO_DEPRECATED
 void KCategorizedView::rowsRemoved(const QModelIndex &parent,
                                    int start,
                                    int end)
@@ -1446,6 +1512,7 @@ void KCategorizedView::rowsRemoved(const QModelIndex &parent,
     Q_UNUSED(start);
     Q_UNUSED(end);
 }
+#endif
 
 void KCategorizedView::slotLayoutChanged()
 {
