@@ -37,13 +37,12 @@
 #include <QtCore/QFile>
 #include <QtCore/QList>
 #include <QtCore/QDateTime>
+#include <QtCore/QCoreApplication>
 
-#include <kapplication.h>
 #include <kcrash.h>
 #include <kconfig.h>
 #include <kconfiggroup.h>
 #include <kde_file.h>
-#include <kdesu/client.h>
 #include <klocale.h>
 
 #include "kremoteencoding.h"
@@ -76,7 +75,8 @@ namespace KIO {
 class SlaveBasePrivate {
 public:
     SlaveBase* q;
-    SlaveBasePrivate(SlaveBase* owner): q(owner) {}
+    SlaveBasePrivate(SlaveBase* owner): q(owner), m_passwdServer(0) {}
+    ~SlaveBasePrivate() { delete m_passwdServer; }
 
     UDSEntryList pendingListEntries;
     int listEntryCurrentSize;
@@ -106,6 +106,8 @@ public:
     enum { Idle, InsideMethod, FinishedCalled, ErrorCalled } m_state;
     QByteArray timeoutData;
 
+    KPasswdServer* m_passwdServer;
+
     // Reconstructs configGroup from configData and mIncomingMetaData
     void rebuildConfig()
     {
@@ -128,12 +130,22 @@ public:
             kWarning(7019) << cmdName << "did not call finished() or error()! Please fix the KIO slave.";
         }
     }
+
     void verifyErrorFinishedNotCalled(const char* cmdName)
     {
         if (m_state == FinishedCalled || m_state == ErrorCalled) {
             kWarning(7019) << cmdName << "called finished() or error(), but it's not supposed to! Please fix the KIO slave.";
         }
     }
+
+    KPasswdServer* passwdServer()
+    {
+        if (!m_passwdServer) {
+            m_passwdServer = new KPasswdServer;
+        }
+
+        return m_passwdServer;
+    }
 };
 
 }
@@ -248,6 +260,7 @@ SlaveBase::~SlaveBase()
 {
     delete d->configGroup;
     delete d->config;
+    delete d->remotefile;
     delete d;
     s_protocol = "";
 }
@@ -293,16 +306,22 @@ void SlaveBase::dispatchLoop()
                 closeConnection();
                 connectSlave(d->poolSocket);
             } else {
-                return;
+                break;
             }
         }
 
         //I think we get here when we were killed in dispatch() and not in select()
         if (wasKilled()) {
-            kDebug(7019)<<" dispatchLoop() slave was killed, returning";
-            return;
+            kDebug(7019) << "slave was killed, returning";
+            break;
         }
+
+        // execute deferred deletes
+        QCoreApplication::sendPostedEvents(NULL, QEvent::DeferredDelete);
     }
+
+    // execute deferred deletes
+    QCoreApplication::sendPostedEvents(NULL, QEvent::DeferredDelete);
 }
 
 void SlaveBase::connectSlave(const QString &address)
@@ -311,7 +330,7 @@ void SlaveBase::connectSlave(const QString &address)
 
     if (!d->appConnection.inited())
     {
-        kDebug(7019) << "SlaveBase: failed to connect to" << address << endl
+        kDebug(7019) << "failed to connect to" << address << endl
 		      << "Reason:" << d->appConnection.errorString();
         exit();
         return;
@@ -375,11 +394,11 @@ void SlaveBase::sendAndKeepMetaData()
 
 KRemoteEncoding *SlaveBase::remoteEncoding()
 {
-   if (d->remotefile != 0)
+   if (d->remotefile)
       return d->remotefile;
 
-   QByteArray charset = metaData("Charset").toLatin1();
-   return d->remotefile = new KRemoteEncoding( charset );
+   const QByteArray charset (metaData(QLatin1String("Charset")).toLatin1());
+   return (d->remotefile = new KRemoteEncoding( charset ));
 }
 
 void SlaveBase::data( const QByteArray &data )
@@ -816,26 +835,40 @@ void SlaveBase::slave_status()
 
 void SlaveBase::reparseConfiguration()
 {
+    delete d->remotefile;
+    d->remotefile = 0;
 }
 
 bool SlaveBase::openPasswordDialog( AuthInfo& info, const QString &errorMsg )
 {
-    const long windowId = metaData("window-id").toLong();
-    const unsigned long userTimestamp = metaData("user-timestamp").toULong();
+    const long windowId = metaData(QLatin1String("window-id")).toLong();
+    const unsigned long userTimestamp = metaData(QLatin1String("user-timestamp")).toULong();
     QString errorMessage;
-    if (metaData("no-auth-prompt").toLower() == "true") {
+    if (metaData(QLatin1String("no-auth-prompt")).compare(QLatin1String("true"), Qt::CaseInsensitive) == 0) {
         errorMessage = QLatin1String("<NoAuthPrompt>");
     } else {
         errorMessage = errorMsg;
     }
 
-    KPasswdServer srv;
-    qlonglong seqNr = srv.queryAuthInfo(info, errorMessage, windowId,
-                                        SlaveBasePrivate::s_seqNr, userTimestamp);
-    if (seqNr > 0) {
-        SlaveBasePrivate::s_seqNr = seqNr;
-        if (info.isModified()) {
-            return true;
+    AuthInfo dlgInfo (info);
+    // Make sure the modified flag is not set.
+    dlgInfo.setModified(false);
+    // Prevent queryAuthInfo from caching the user supplied password since
+    // we need the ioslaves to first authenticate against the server with
+    // it to ensure it is valid.
+    dlgInfo.setExtraField(QLatin1String("skip-caching-on-query"), true);
+
+    KPasswdServer* passwdServer = d->passwdServer();
+
+    if (passwdServer) {
+        qlonglong seqNr = passwdServer->queryAuthInfo(dlgInfo, errorMessage, windowId,
+                                                      SlaveBasePrivate::s_seqNr, userTimestamp);
+        if (seqNr > 0) {
+            SlaveBasePrivate::s_seqNr = seqNr;
+            if (dlgInfo.isModified()) {
+                info = dlgInfo;
+                return true;
+            }
         }
     }
 
@@ -1168,8 +1201,10 @@ void SlaveBase::dispatch( int command, const QByteArray &data )
 
 bool SlaveBase::checkCachedAuthentication( AuthInfo& info )
 {
-    return KPasswdServer().checkAuthInfo(info, metaData("window-id").toLong(),
-                                         metaData("user-timestamp").toULong());
+    KPasswdServer* passwdServer = d->passwdServer();
+    return (passwdServer &&
+            passwdServer->checkAuthInfo(info, metaData(QLatin1String("window-id")).toLong(),
+                                        metaData(QLatin1String("user-timestamp")).toULong()));
 }
 
 void SlaveBase::dispatchOpenCommand( int command, const QByteArray &data )
@@ -1206,14 +1241,20 @@ void SlaveBase::dispatchOpenCommand( int command, const QByteArray &data )
 
 bool SlaveBase::cacheAuthentication( const AuthInfo& info )
 {
-    KPasswdServer().addAuthInfo(info, metaData("window-id").toLongLong());
+    KPasswdServer* passwdServer = d->passwdServer();
+
+    if (!passwdServer) {
+        return false;
+    }
+
+    passwdServer->addAuthInfo(info, metaData(QLatin1String("window-id")).toLongLong());
     return true;
 }
 
 int SlaveBase::connectTimeout()
 {
     bool ok;
-    QString tmp = metaData("ConnectTimeout");
+    QString tmp = metaData(QLatin1String("ConnectTimeout"));
     int result = tmp.toInt(&ok);
     if (ok)
        return result;
@@ -1223,7 +1264,7 @@ int SlaveBase::connectTimeout()
 int SlaveBase::proxyConnectTimeout()
 {
     bool ok;
-    QString tmp = metaData("ProxyConnectTimeout");
+    QString tmp = metaData(QLatin1String("ProxyConnectTimeout"));
     int result = tmp.toInt(&ok);
     if (ok)
        return result;
@@ -1234,7 +1275,7 @@ int SlaveBase::proxyConnectTimeout()
 int SlaveBase::responseTimeout()
 {
     bool ok;
-    QString tmp = metaData("ResponseTimeout");
+    QString tmp = metaData(QLatin1String("ResponseTimeout"));
     int result = tmp.toInt(&ok);
     if (ok)
        return result;
@@ -1245,7 +1286,7 @@ int SlaveBase::responseTimeout()
 int SlaveBase::readTimeout()
 {
     bool ok;
-    QString tmp = metaData("ReadTimeout");
+    QString tmp = metaData(QLatin1String("ReadTimeout"));
     int result = tmp.toInt(&ok);
     if (ok)
        return result;
