@@ -36,6 +36,7 @@
 #include <plasma/plasma.h>
 #include <plasma/animator.h>
 #include <plasma/version.h>
+#include <plasma/framesvg.h>
 
 class QWidget;
 
@@ -54,6 +55,7 @@ class Extender;
 class ExtenderItem;
 class Package;
 
+
 /**
  * @class Applet plasma/applet.h <Plasma/Applet>
  *
@@ -70,20 +72,26 @@ class Package;
  * of scripting support for each applet, providing access to the associated
  * plasmoid package (if any) and access to configuration data.
  *
- * See techbase.kde.org for tutorial on writing Applets using this class.
+ * See techbase.kde.org for tutorials on writing Applets using this class.
  */
 class PLASMA_EXPORT Applet : public QGraphicsWidget
 {
     Q_OBJECT
     Q_PROPERTY(bool hasConfigurationInterface READ hasConfigurationInterface)
     Q_PROPERTY(QString name READ name)
+    Q_PROPERTY(QString pluginName READ pluginName)
     Q_PROPERTY(QString category READ category)
     Q_PROPERTY(ImmutabilityType immutability READ immutability WRITE setImmutability)
     Q_PROPERTY(bool hasFailedToLaunch READ hasFailedToLaunch WRITE setFailedToLaunch)
-    Q_PROPERTY(bool isBusy READ isBusy WRITE setBusy)
+    Q_PROPERTY(bool isBusy READ isBusy WRITE setBusy) //KDE5: remove
+    Q_PROPERTY(bool busy READ isBusy WRITE setBusy)
     Q_PROPERTY(bool configurationRequired READ configurationRequired WRITE setConfigurationRequired)
     Q_PROPERTY(QRectF geometry READ geometry WRITE setGeometry)
     Q_PROPERTY(bool shouldConserveResources READ shouldConserveResources)
+    Q_PROPERTY(uint id READ id)
+    Q_PROPERTY(bool userConfiguring READ isUserConfiguring)
+    Q_PROPERTY(BackgroundHints backgroundHints READ backgroundHints WRITE setBackgroundHints)
+    Q_ENUMS(BackgroundHints)
 
     public:
         typedef QList<Applet*> List;
@@ -106,7 +114,7 @@ class PLASMA_EXPORT Applet : public QGraphicsWidget
         ~Applet();
 
         /**
-         * @return a package structure representing a Theme
+         * @return a package structure representing an Applet
          */
         static PackageStructure::Ptr packageStructure();
 
@@ -229,6 +237,16 @@ class PLASMA_EXPORT Applet : public QGraphicsWidget
          */
         QPoint popupPosition(const QSize &s) const;
 
+        /**
+         * @since 4.4
+         * Reccomended position for a popup window like a menu or a tooltip
+         * given its size
+         * @param s size of the popup
+         * @param alignment alignment of the popup, valid flags are Qt::AlignLeft, Qt::AlignRight and Qt::AlignCenter
+         * @returns reccomended position
+         */
+        QPoint popupPosition(const QSize &s, Qt::AlignmentFlag alignment) const;
+
         /**
          * Called when any of the geometry constraints have been updated.
          * This method calls constraintsEvent, which may be reimplemented,
@@ -269,6 +287,7 @@ class PLASMA_EXPORT Applet : public QGraphicsWidget
 
         /**
          * Returns a list of all known applets.
+         * This may skip applets based on security settings and ExcludeCategories in the application's config.
          *
          * @param category Only applets matchin this category will be returned.
          *                 Useful in conjunction with knownCategories.
@@ -284,7 +303,7 @@ class PLASMA_EXPORT Applet : public QGraphicsWidget
          * @return list of applets
          **/
         static KPluginInfo::List listAppletInfo(const QString &category = QString(),
-                                              const QString &parentApp = QString());
+                                                const QString &parentApp = QString());
 
         /**
          * Returns a list of all known applets associated with a certain mimetype.
@@ -294,8 +313,15 @@ class PLASMA_EXPORT Applet : public QGraphicsWidget
         static KPluginInfo::List listAppletInfoForMimetype(const QString &mimetype);
 
         /**
-         * Returns a list of all the categories used by
-         * installed applets.
+         * Returns a list of all known applets associated with a certain URL.
+         *
+         * @since 4.4
+         * @return list of applets
+         **/
+        static KPluginInfo::List listAppletInfoForUrl(const QUrl &url);
+
+        /**
+         * Returns a list of all the categories used by installed applets.
          *
          * @param parentApp the application to filter applets on. Uses the
          *                  X-KDE-ParentApp entry (if any) in the plugin info.
@@ -308,6 +334,37 @@ class PLASMA_EXPORT Applet : public QGraphicsWidget
         static QStringList listCategories(const QString &parentApp = QString(),
                                           bool visibleOnly = true);
 
+        /**
+         * Sets the list of custom categories that are used in addition to the default
+         * set of categories known to libplasma for Applets.
+         * @param categories a list of categories
+         * @since 4.3
+         */
+        void setCustomCategories(const QStringList &categories);
+
+        /**
+         * @return the list of custom categories known to libplasma
+         * @since 4.3
+         */
+        QStringList customCategories();
+
+        /**
+         * Attempts to load an applet from a package
+         *
+         * Returns a pointer to the applet if successful.
+         * The caller takes responsibility for the applet, including
+         * deleting it when no longer needed.
+         *
+         * @param path the path to the package
+         * @param appletId unique ID to assign the applet, or zero to have one
+         *        assigned automatically.
+         * @param args to send the applet extra arguments
+         * @return a pointer to the loaded applet, or 0 on load failure
+         * @since 4.3
+         **/
+        static Applet *loadPlasmoid(const QString &path, uint appletId = 0,
+                                    const QVariantList &args = QVariantList());
+
         /**
          * Attempts to load an applet
          *
@@ -448,7 +505,7 @@ class PLASMA_EXPORT Applet : public QGraphicsWidget
         /**
          * Returns the QAction with the given name from our collection
          */
-        QAction *action(QString name) const;
+        Q_INVOKABLE QAction *action(QString name) const;
 
         /**
          * Adds the action to our collection under the given name
@@ -528,7 +585,7 @@ class PLASMA_EXPORT Applet : public QGraphicsWidget
         virtual void removeAssociatedWidget(QWidget *widget);
 
         /**
-         * Gets called when and extender item has to be initialized after a plasma restart. If you
+         * Gets called when an extender item has to be initialized after a plasma restart. If you
          * create ExtenderItems in your applet, you should implement this function to again create
          * the widget that should be shown in this extender item. This function might look something
          * like this:
@@ -557,6 +614,31 @@ class PLASMA_EXPORT Applet : public QGraphicsWidget
                         const QString &serviceId = QString(),
                         uint appletId = 0);
 
+        /**
+         * @param parent the QGraphicsItem this applet is parented to
+         * @param info the plugin information object for this Applet
+         * @param appletId a unique id used to differentiate between multiple
+         *      instances of the same Applet type
+         * @since 4.6
+         */
+        explicit Applet(const KPluginInfo &info, QGraphicsItem *parent = 0, uint appletId = 0);
+
+        /**
+         * @param parent the QGraphicsItem this applet is parented to
+         * @param serviceId the name of the .desktop file containing the
+         *      information about the widget
+         * @param appletId a unique id used to differentiate between multiple
+         *      instances of the same Applet type
+         * @param args  a list of strings containing two entries: the service id
+         *      and the applet id
+         * @since 4.3
+         */
+        explicit Applet(QGraphicsItem *parent,
+                        const QString &serviceId,
+                        uint appletId,
+                        const QVariantList &args);
+
+
         /**
          * @return true if destroy() was called; useful for Applets which should avoid
          * certain tasks if they are about to be deleted permanently
@@ -568,25 +650,103 @@ class PLASMA_EXPORT Applet : public QGraphicsWidget
          * parented to the supplied widget. Ownership of the widgets is passed
          * to the parent widget.
          *
+         * Typically one would add custom pages to the config dialog @p parent
+         * and then connect to the applyClicked() and okClicked() signals
+         * or @p parent to react on config changes:
+         *
+         * @code
+         * connect(parent, SIGNAL(applyClicked()), this, SLOT(myConfigAccepted()));
+         * connect(parent, SIGNAL(okClicked()), this, SLOT(myConfigAccepted()));
+         * @endcode
+         *
+         * With this approach it makes sense to store the custom pages in member
+         * variables to make their fields accessible from the myConfigAccepted()
+         * slot.
+         *
+         * Use config() to store your configuration.
+         *
          * @param parent the dialog which is the parent of the configuration
          *               widgets
          */
         virtual void createConfigurationInterface(KConfigDialog *parent);
 
+        /**
+         * Returns true if the applet is allowed to perform functions covered by the given constraint
+         * eg. hasAuthorization("FileDialog") returns true if applets are allowed to show filedialogs.
+         * @since 4.3
+         */
+        bool hasAuthorization(const QString &constraint) const;
+
+        /**
+         * Sets an application associated to this applet, that will be
+         * regarded as a full view of what is represented in the applet
+         *
+         * @param string the name of the application. it can be
+         *      \li a name understood by KService::serviceByDesktopName
+         *        (e.g. "konqueror")
+         *      \li a command in $PATH
+         *      \li or an absolute path to an executable
+         * @since 4.4
+         */
+        void setAssociatedApplication(const QString &string);
+
+        /**
+         * Sets a list of urls associated to this application,
+         * they will be used as parameters for the associated application
+         * @see setAssociatedApplication()
+         *
+         * @param urls
+         */
+        void setAssociatedApplicationUrls(const KUrl::List &urls);
+
+        /**
+         * @return the application associated to this applet
+         * @since 4.4
+         */
+        QString associatedApplication() const;
+
+        /**
+         * @return the urls associated to this applet
+         * @since 4.4
+         */
+        KUrl::List associatedApplicationUrls() const;
+
+        /**
+         * @return true if the applet has a valid associated application or urls
+         * @since 4.4
+         */
+        bool hasValidAssociatedApplication() const;
+
     Q_SIGNALS:
         /**
          * This signal indicates that an application launch, window
          * creation or window focus event was triggered. This is used, for instance,
-         * to ensure that the Dashboard view in Plasma hides when such an event is
+         * to ensure that the Dashboard view in Plasma Desktop hides when such an event is
          * triggered by an item it is displaying.
          */
         void releaseVisualFocus();
 
+#if QT_VERSION >= 0x040700
+    protected:
+        void geometryChanged(); // in QGraphicsWidget now; preserve BC
+#else
         /**
          * Emitted whenever the applet makes a geometry change, so that views
          * can coordinate themselves with these changes if they desire.
          */
         void geometryChanged();
+#endif
+
+    Q_SIGNALS:
+        /**
+         * Emitted when the user completes a transformation of the applet.
+         */
+        void appletTransformedByUser();
+
+        /**
+         * Emitted when the applet changes its own geometry or transform.
+         */
+        void appletTransformedItself();
 
         /**
          * Emitted by Applet subclasses when they change a sizeHint and wants to announce the change
@@ -609,16 +769,42 @@ class PLASMA_EXPORT Applet : public QGraphicsWidget
          */
         void activate();
 
+        /**
+         * Emitted when the user clicked on a button of the message overlay
+         *
+         * @see showMessage
+         * @see Plasma::MessageButton
+         * @since 4.3
+         */
+        void messageButtonPressed(const Plasma::MessageButton button);
+
         /**
          * Emitted when the applet is deleted
          */
         void appletDestroyed(Plasma::Applet *applet);
 
+        /**
+         * Emitted when the applet status changes
+         * @since 4.4
+         */
+        void newStatus(Plasma::ItemStatus status);
+
+        /**
+         * Emitted when an ExtenderItem in a scripting applet needs to be initialized
+         */
+        void extenderItemRestored(Plasma::ExtenderItem *item);
+
+        /**
+         * Emitted when the immutability changes
+         * @since 4.4
+         */
+        void immutabilityChanged(Plasma::ImmutabilityType immutable);
+
     public Q_SLOTS:
         /**
          * Sets the immutability type for this applet (not immutable,
          * user immutable or system immutable)
-         * @arg immutable the new immutability type of this applet
+         * @param immutable the new immutability type of this applet
          */
         void setImmutability(const ImmutabilityType immutable);
 
@@ -638,9 +824,27 @@ class PLASMA_EXPORT Applet : public QGraphicsWidget
          * instead. A good example of when this isn't plausible is
          * when using a dialog prepared by another library, such
          * as KPropertiesDialog from libkfile.
+         * You probably want to call showConfigurationInterface(QWidget*)
+         * with the custom widget you created to actually show your interface
          */
         virtual void showConfigurationInterface();
 
+        /**
+         * Actually show your custom configuration interface
+         * Use this only if you reimplemented showConfigurationInterface()
+         *
+         * @param widget the widget representing your configuration interface
+         *
+         * @since 4.5
+         */
+        void showConfigurationInterface(QWidget *widget);
+
+        /**
+         * @return true when the configuration interface is being shown
+         * @since 4.5
+         */
+        bool isUserConfiguring() const;
+
         /**
          * Causes this applet to raise above all other applets.
          */
@@ -670,7 +874,7 @@ class PLASMA_EXPORT Applet : public QGraphicsWidget
         virtual void init();
 
         /**
-         * Called when applet configuration values has changed.
+         * Called when applet configuration values have changed.
          */
         virtual void configChanged();
 
@@ -680,6 +884,46 @@ class PLASMA_EXPORT Applet : public QGraphicsWidget
          */
         void setBusy(bool busy);
 
+        /**
+         * @return the list of arguments which the applet was called with
+         * @since KDE4.3
+         */
+        QVariantList startupArguments() const;
+
+        /**
+         * @return the status of the applet
+         * @since 4.4
+         */
+        ItemStatus status() const;
+
+        /**
+         * sets the status for this applet
+         * @since 4.4
+         */
+        void setStatus(const ItemStatus stat);
+
+        /**
+         * Publishes and optionally announces this applet on the network for remote access.
+         * @param methods the methods to use for announcing this applet.
+         * @param resourceName the name under which this applet will be published (has to be unique
+         * for each machine)
+         */
+        void publish(Plasma::AnnouncementMethods methods, const QString &resourceName);
+
+        void unpublish();
+
+        bool isPublished() const;
+
+        /**
+         * Open the application associated to this applet, if it's not set
+         * but some urls are, open those urls with the proper application
+         * for their mimetype
+         * @see setAssociatedApplication()
+         * @see setAssociatedApplicationUrls()
+         * @since 4.4
+         */
+        void runAssociatedApplication();
+
     protected:
         /**
          * This constructor is to be used with the plugin loading systems
@@ -743,9 +987,31 @@ class PLASMA_EXPORT Applet : public QGraphicsWidget
         void setConfigurationRequired(bool needsConfiguring, const QString &reason = QString());
 
         /**
-         * Called when any of the geometry constraints have been updated.
+         * Shows a message as an overlay of the applet: the message has an
+         * icon, text and (optional) buttons
          *
-         * This is always called prior to painting and should be used as an
+         * @param icon the icon that will be shown
+         * @param message the message string that will be shown.
+         *                If the message is empty nothng will be shown
+         *                and if there was a message already it will be hidden
+         * @param buttons an OR combination of all the buttons needed
+         *
+         * @see Plasma::MessageButtons
+         * @see messageButtonPressed
+         * @since 4.3
+         */
+        void showMessage(const QIcon &icon, const QString &message, const Plasma::MessageButtons buttons);
+
+        /**
+         * Called when any of the constraints for the applet have been updated. These constraints
+         * range from notifying when the applet has officially "started up" to when geometry changes
+         * to when the form factor changes.
+         *
+         * Each constraint that has been changed is passed in the constraints flag.
+         * All of the constraints and how they work is documented in the @see Plasma::Constraints
+         * enumeration.
+         *
+         * On applet creation, this is always called prior to painting and can be used as an
          * opportunity to layout the widget, calculate sizings, etc.
          *
          * Do not call update() from this method; an update() will be triggered
@@ -840,17 +1106,30 @@ class PLASMA_EXPORT Applet : public QGraphicsWidget
          */
         void timerEvent (QTimerEvent *event);
 
+
     private:
+        /**
+         * @internal This constructor is to be used with the Package loading system.
+         *
+         * @param parent a QObject parent; you probably want to pass in 0
+         * @param args a list of strings containing two entries: the service id
+         *      and the applet id
+         * @since 4.3
+         */
+        Applet(const QString &packagePath, uint appletId, const QVariantList &args);
+
         Q_PRIVATE_SLOT(d, void setFocus())
-        Q_PRIVATE_SLOT(d, void checkImmutability())
         Q_PRIVATE_SLOT(d, void themeChanged())
-        Q_PRIVATE_SLOT(d, void appletAnimationComplete(QGraphicsItem *item,
-                                                       Plasma::Animator::Animation anim))
+        Q_PRIVATE_SLOT(d, void cleanUpAndDelete())
         Q_PRIVATE_SLOT(d, void selectItemToDestroy())
         Q_PRIVATE_SLOT(d, void updateRect(const QRectF& rect))
         Q_PRIVATE_SLOT(d, void destroyMessageOverlay())
-        Q_PRIVATE_SLOT(d, void clearShortcutEditorPtr())
         Q_PRIVATE_SLOT(d, void configDialogFinished())
+        Q_PRIVATE_SLOT(d, void updateShortcuts())
+        Q_PRIVATE_SLOT(d, void publishCheckboxStateChanged(int state))
+        Q_PRIVATE_SLOT(d, void globalShortcutChanged())
+        Q_PRIVATE_SLOT(d, void propagateConfigChanged())
+        Q_PRIVATE_SLOT(d, void handleDisappeared(AppletHandle *handle))
 
         /**
          * Reimplemented from QGraphicsItem
@@ -867,10 +1146,16 @@ class PLASMA_EXPORT Applet : public QGraphicsWidget
         friend class AppletScript;
         friend class AppletHandle;
         friend class AppletPrivate;
+        friend class AccessAppletJobPrivate;
+        friend class PluginLoader;
         friend class PopupApplet;
         friend class PopupAppletPrivate;
+        friend class AssociatedApplicationManager;
 
         friend class Extender;
+        friend class ExtenderGroup;
+        friend class ExtenderGroupPrivate;
+        friend class ExtenderPrivate;
         friend class ExtenderItem;
 };
 
