@@ -29,6 +29,8 @@
 #include <QTouchEvent>
 #include <QMatrix>
 #include <QTransform>
+#include <QWeakPointer>
+#include <QPropertyAnimation>
 
 #include <kcolorscheme.h>
 #include <kglobalsettings.h>
@@ -45,6 +47,7 @@
 #include "corona.h"
 #include "paintutils.h"
 #include "theme.h"
+#include "tooltipmanager.h"
 #include "view.h"
 #include "framesvg.h"
 
@@ -56,15 +59,14 @@ qreal _k_pointAngle(QPointF point);
 QPointF _k_rotatePoint(QPointF point, qreal angle);
 QPointF _k_projectPoint(QPointF point, QPointF v);
 
-AppletHandle::AppletHandle(Containment *parent, Applet *applet, const QPointF &hoverPos)
+AppletHandle::AppletHandle(Containment *containment, Applet *applet, const QPointF &hoverPos)
     : QGraphicsObject(applet),
       m_pressedButton(NoButton),
-      m_containment(parent),
+      m_containment(containment),
       m_applet(applet),
       m_iconSize(KIconLoader::SizeSmall),
       m_opacity(0.0),
-      m_anim(FadeIn),
-      m_animId(0),
+      m_animType(FadeIn),
       m_backgroundBuffer(0),
       m_currentView(applet->view()),
       m_entryPos(hoverPos),
@@ -101,9 +103,11 @@ AppletHandle::AppletHandle(Containment *parent, Applet *applet, const QPointF &h
     m_configureIcons = new Svg(this);
     m_configureIcons->setImagePath("widgets/configuration-icons");
     m_configureIcons->setContainsMultipleImages(true);
+    connect(m_configureIcons, SIGNAL(repaintNeeded()), this, SLOT(scheduleUpdate()));
 
     m_background = new FrameSvg(this);
     m_background->setImagePath("widgets/background");
+    connect(m_background, SIGNAL(repaintNeeded()), this, SLOT(scheduleUpdate()));
     m_applet->installSceneEventFilter(this);
 }
 
@@ -113,6 +117,11 @@ AppletHandle::~AppletHandle()
     delete m_backgroundBuffer;
 }
 
+bool AppletHandle::shown() const
+{
+    return !m_hoverTimer->isActive();
+}
+
 Applet *AppletHandle::applet() const
 {
     return m_applet;
@@ -187,7 +196,7 @@ void AppletHandle::paint(QPainter *painter, const QStyleOptionGraphicsItem *opti
 
     //kDebug() << m_opacity << m_anim << FadeOut;
     if (qFuzzyCompare(m_opacity + 1.0, 1.0)) {
-        if (m_anim == FadeOut) {
+        if (m_animType == FadeOut) {
             //kDebug() << "WOOOOOOOOO";
             QTimer::singleShot(0, this, SLOT(emitDisappear()));
         }
@@ -215,28 +224,35 @@ void AppletHandle::paint(QPainter *painter, const QStyleOptionGraphicsItem *opti
                            int(m_decorationRect.height()) + m_iconSize * 5 + 1);
     const QSize iconSize(KIconLoader::SizeSmall, KIconLoader::SizeSmall);
 
+    bool isRunning = false;
+    if (m_anim.data()) {
+        isRunning = m_anim.data()->state() == QAbstractAnimation::Running ? \
+                    true : false;
+    }
+
     //regenerate our buffer?
-    if (m_animId > 0 || !m_backgroundBuffer || m_backgroundBuffer->size() != pixmapSize) {
+    if (isRunning || !m_backgroundBuffer || m_backgroundBuffer->size() != pixmapSize) {
         QColor transparencyColor = Qt::black;
         transparencyColor.setAlphaF(qMin(m_opacity, qreal(0.99)));
 
         QLinearGradient g(QPoint(0, 0), QPoint(m_decorationRect.width(), 0));
         //fading out panel
-        if (m_rect.height() > qreal(minimumHeight()) * 1.25) {
+        if (m_applet && m_applet->backgroundHints() != Plasma::Applet::NoBackground &&
+            m_rect.height() > qreal(minimumHeight()) * 1.25) {
             if (m_buttonsOnRight) {
                 qreal opaquePoint =
                     (m_background->marginSize(LeftMargin) - translation) / m_decorationRect.width();
                 //kDebug() << "opaquePoint" << opaquePoint
                 //         << m_background->marginSize(LeftMargin) << m_decorationRect.width();
                 g.setColorAt(0.0, Qt::transparent);
-                g.setColorAt(qMax(0.0, opaquePoint - 0.05), Qt::transparent);
+                g.setColorAt(qMax(0.0, opaquePoint - 0.01), Qt::transparent); //krazy:exclude=qminmax
                 g.setColorAt(opaquePoint, transparencyColor);
                 g.setColorAt(1.0, transparencyColor);
             } else {
                 qreal opaquePoint =
                     1 - ((m_background->marginSize(RightMargin) + translation) / m_decorationRect.width());
                 g.setColorAt(1.0, Qt::transparent);
-                g.setColorAt(opaquePoint + 0.05, Qt::transparent);
+                g.setColorAt(opaquePoint + 0.01, Qt::transparent);
                 g.setColorAt(qMax(qreal(0), opaquePoint), transparencyColor);
                 g.setColorAt(0.0, transparencyColor);
             }
@@ -375,6 +391,11 @@ void AppletHandle::emitDisappear()
     emit disappearDone(this);
 }
 
+void AppletHandle::scheduleUpdate()
+{
+    update();
+}
+
 AppletHandle::ButtonType AppletHandle::mapToButton(const QPointF &point) const
 {
     int iconMargin = m_iconSize / 2;
@@ -464,8 +485,8 @@ void AppletHandle::mousePressEvent(QGraphicsSceneMouseEvent *event)
         //set mousePos to the position in the applet, in screencoords, so it becomes easy
         //to reposition the toplevel view to the correct position.
         if (m_currentView && m_applet) {
-            QPoint localpos = m_currentView->mapFromScene(m_applet->scenePos());
-            m_mousePos = event->screenPos() - m_currentView->mapToGlobal(localpos);
+            QPoint localpos = m_currentView.data()->mapFromScene(m_applet->scenePos());
+            m_mousePos = event->screenPos() - m_currentView.data()->mapToGlobal(localpos);
         }
         return;
     }
@@ -480,7 +501,7 @@ bool AppletHandle::leaveCurrentView(const QPoint &pos) const
             //is this widget a plasma view, a different view then our current one,
             //AND not a dashboardview?
             Plasma::View *v = qobject_cast<Plasma::View *>(widget);
-            if (v && v != m_currentView && v->containment() != m_containment) {
+            if (v && v != m_currentView.data() && v->containment() != m_containment) {
                 return true;
             }
         }
@@ -602,7 +623,7 @@ void AppletHandle::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
     if (m_pressedButton == MoveButton) {
         if (leaveCurrentView(event->screenPos())) {
             Plasma::View *v = Plasma::View::topLevelViewAt(event->screenPos());
-            if (v && v != m_currentView) {
+            if (v && v != m_currentView.data()) {
                 Containment *c = v->containment();
                 if (c) {
                     QPoint pos = v->mapFromGlobal(event->screenPos());
@@ -789,7 +810,7 @@ void AppletHandle::switchContainment(Containment *containment, const QPointF &po
     applet->disconnect(this); //make sure the applet doesn't tell us to do anything
     //applet->setZValue(m_zValue);
     containment->addApplet(applet, containment->mapFromScene(pos), false);
-    update();
+    deleteLater();
 }
 
 void AppletHandle::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
@@ -797,12 +818,15 @@ void AppletHandle::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
     Q_UNUSED(event);
     //kDebug() << "hover enter";
 
-    //if a disappear was scheduled stop the timer
-    if (m_leaveTimer->isActive()) {
-        m_leaveTimer->stop();
+    if (m_applet) {
+        ToolTipManager::self()->hide(m_applet);
     }
-    // if we're already fading out, fade back in
-    else if (m_animId != 0 && m_anim == FadeOut) {
+
+    if (m_leaveTimer->isActive()) {
+        //if a disappear was scheduled stop the timer
+        m_leaveTimer->stop();
+    } else if (!m_anim.data() && m_animType == FadeOut) {
+        // if we're already fading out, fade back in
         startFading(FadeIn, m_entryPos, true);
     }
 }
@@ -841,23 +865,51 @@ bool AppletHandle::sceneEventFilter(QGraphicsItem *watched, QEvent *event)
 {
     if (watched == m_applet && event->type() == QEvent::GraphicsSceneHoverLeave) {
         hoverLeaveEvent(static_cast<QGraphicsSceneHoverEvent*>(event));
+    } else if (watched == m_applet && event->type() == QEvent::GraphicsSceneMousePress) {
+        QGraphicsSceneMouseEvent *me = static_cast<QGraphicsSceneMouseEvent *>(event);
+
+        QPointF pos = mapFromScene(me->scenePos());
+        if (m_applet->contentsRect().contains(pos)) {
+            return false;
+        }
+
+        me->setPos(pos);
+        mousePressEvent(me);
+        return true;
+    } else if (watched == m_applet && event->type() == QEvent::GraphicsSceneMouseRelease) {
+        if (m_pressedButton == NoButton) {
+            return false;
+        }
+
+        QGraphicsSceneMouseEvent *me = static_cast<QGraphicsSceneMouseEvent *>(event);
+
+        QPointF pos = mapFromScene(me->scenePos());
+        me->setPos(pos);
+        mouseReleaseEvent(me);
+        return true;
+    } else if (watched == m_applet && event->type() == QEvent::GraphicsSceneMouseMove) {
+        if (m_pressedButton == NoButton) {
+            return false;
+        }
+
+        QGraphicsSceneMouseEvent *me = static_cast<QGraphicsSceneMouseEvent *>(event);
+
+        QPointF pos = mapFromScene(me->scenePos());
+
+
+        me->setPos(pos);
+        mouseMoveEvent(me);
+        return true;
     }
 
     return false;
 }
 
-void AppletHandle::fadeAnimation(qreal progress)
+void AppletHandle::setFadeAnimation(qreal progress)
 {
-    //qreal endOpacity = (m_anim == FadeIn) ? 1.0 : -1.0;
-    if (m_anim == FadeIn) {
-        m_opacity = progress;
-    } else {
-        m_opacity = 1 - progress;
-    }
-
+    m_opacity = progress;
     //kDebug() << "progress" << progress << "m_opacity" << m_opacity << m_anim << "(" << FadeIn << ")";
     if (qFuzzyCompare(progress, qreal(1.0))) {
-        m_animId = 0;
         delete m_backgroundBuffer;
         m_backgroundBuffer = 0;
     }
@@ -865,6 +917,11 @@ void AppletHandle::fadeAnimation(qreal progress)
     update();
 }
 
+qreal AppletHandle::fadeAnimation() const
+{
+    return m_opacity;
+}
+
 void AppletHandle::hoverTimeout()
 {
     startFading(FadeIn, m_entryPos);
@@ -896,16 +953,23 @@ void AppletHandle::setHoverPos(const QPointF &hoverPos)
 
 void AppletHandle::startFading(FadeType anim, const QPointF &hoverPos, bool preserveSide)
 {
-    if (m_animId != 0) {
-        Animator::self()->stopCustomAnimation(m_animId);
+    QPropertyAnimation *propAnim = m_anim.data();
+
+    if (anim == FadeIn) {
+        if (propAnim) {
+            propAnim->stop();
+        } else {
+            propAnim = new QPropertyAnimation(this, "fadeAnimation", this);
+            m_anim = propAnim;
+        }
     }
 
     m_entryPos = hoverPos;
     qreal time = 100;
 
     if (!m_applet) {
-        m_anim = FadeOut;
-        fadeAnimation(1.0);
+        m_animType = FadeOut;
+        setFadeAnimation(1.0);
         return;
     }
 
@@ -940,7 +1004,7 @@ void AppletHandle::startFading(FadeType anim, const QPointF &hoverPos, bool pres
         }
 
         if (wasOnRight != m_buttonsOnRight &&
-            m_anim == FadeIn &&
+            m_animType == FadeIn &&
             anim == FadeIn &&
             m_opacity <= 1) {
             m_opacity = 0.0;
@@ -951,10 +1015,21 @@ void AppletHandle::startFading(FadeType anim, const QPointF &hoverPos, bool pres
         time *= m_opacity;
     }
 
-    m_anim = anim;
+    if (propAnim) {
+        propAnim->setStartValue(0);
+        propAnim->setEndValue(1);
+        propAnim->setDuration(time);
+    }
+
+    m_animType = anim;
     //kDebug() << "animating for " << time << "ms";
-    m_animId = Animator::self()->customAnimation(80 * (time / 1000.0), (int)time,
-                                                 Animator::EaseInCurve, this, "fadeAnimation");
+    if (m_animType == FadeIn) {
+        propAnim->setDirection(QAbstractAnimation::Forward);
+        propAnim->start();
+    } else if (propAnim) {
+        propAnim->setDirection(QAbstractAnimation::Backward);
+        propAnim->start(QAbstractAnimation::DeleteWhenStopped);
+    }
 }
 
 void AppletHandle::forceDisappear()
@@ -991,29 +1066,32 @@ void AppletHandle::calculateSize()
     qreal marginLeft, marginTop, marginRight, marginBottom;
     m_background->getMargins(marginLeft, marginTop, marginRight, marginBottom);
 
-    if (m_buttonsOnRight) {
-        //put the rect on the right of the applet
-        m_rect = QRectF(m_applet->size().width(), top, handleWidth, handleHeight);
-    } else {
-        //put the rect on the left of the applet
-        m_rect = QRectF(-handleWidth, top, handleWidth, handleHeight);
+    int leftShadowWidth = m_background->elementSize("hint-left-shadow").width();
+    int rightShadowWidth = m_background->elementSize("hint-right-shadow").width();
+
+    //no precise hint? let's go for heuristics
+    if (leftShadowWidth <= 0) {
+        leftShadowWidth = marginLeft/1.6;
+    }
+    if (rightShadowWidth <= 0) {
+        rightShadowWidth = marginRight/1.6;
     }
 
-    if (m_applet->contentsRect().height() > qreal(minimumHeight()) * 1.25) {
-        int addedMargin = marginLeft / 2;
+    if (m_applet->backgroundHints() == Plasma::Applet::NoBackground) {
+        leftShadowWidth = 0;
+        rightShadowWidth = 0;
+    }
+    if (m_buttonsOnRight) {
+        //put the rect on the right of the applet
+        m_rect = QRectF(m_applet->size().width() - rightShadowWidth + HANDLE_MARGIN, top, handleWidth, handleHeight);
+    } else {
+        //put the rect on the left of the applet
+        m_rect = QRectF(-handleWidth + leftShadowWidth - HANDLE_MARGIN, top, handleWidth, handleHeight);
+    }
 
-        // now we check to see if the shape is smaller than the contents,
-        // and that the shape is not just the bounding rect; in those cases
-        // we have a shaped guy and we draw a full panel;
-        // TODO: allow applets to mark when they have translucent areas and
-        //       should therefore skip this test?
-        if (!m_applet->shape().contains(m_applet->contentsRect())) {
-            QPainterPath p;
-            p.addRect(m_applet->boundingRect());
-            if (m_applet->shape() != p) {
-                addedMargin = m_applet->contentsRect().width() / 2;
-            }
-        }
+    if (m_applet->backgroundHints() != Plasma::Applet::NoBackground &&
+        m_applet->contentsRect().height() > qreal(minimumHeight()) * 1.25) {
+        int addedMargin = HANDLE_MARGIN;
 
         if (m_buttonsOnRight) {
             marginLeft += addedMargin;
